from nuclia.sdk.search import NucliaSearch
from nuclia.tests.fixtures import IS_PROD


def test_find(testing_config):
    if IS_PROD:
        assert True
        return
    search = NucliaSearch()
    results = search.find(query="Who is hedy Lamarr?")
    assert len(results.resources.keys()) == 2
    titles = [r.title for r in results.resources.values()]
    assert "Lamarr Lesson plan.pdf" in titles


def test_find_object(testing_config):
    if IS_PROD:
        assert True
        return
    search = NucliaSearch()
    results = search.find(query={"query": "Who is hedy Lamarr?"})
    assert len(results.resources.keys()) == 2
    titles = [r.title for r in results.resources.values()]
    assert "Lamarr Lesson plan.pdf" in titles


def test_ask(testing_config):
    if IS_PROD:
        assert True
        return
    search = NucliaSearch()
    results = search.ask(query="Who is hedy Lamarr?")
    answer = results.answer.decode()
    print("Answer: ", answer)
    assert "Lamarr" in answer


def test_search(testing_config):
    if IS_PROD:
        assert True
        return
    search = NucliaSearch()
    results = search.search(query="Who is hedy Lamarr?")
    assert len(results.resources.keys()) == 2
    titles = [r.title for r in results.resources.values()]
    assert "Lamarr Lesson plan.pdf" in titles


def test_search_object(testing_config):
    if IS_PROD:
        assert True
        return
    search = NucliaSearch()
    results = search.search(query={"query": "Who is hedy Lamarr?"})
    assert len(results.resources.keys()) == 2
    titles = [r.title for r in results.resources.values()]
    assert "Lamarr Lesson plan.pdf" in titles


def test_filters(testing_config):
    if IS_PROD:
        assert True
        return
    search = NucliaSearch()
    results = search.find(
        query="Who is hedy Lamarr?", filters=["/icon/application/pdf"]
    )
    assert len(results.resources.keys()) == 1



from nuclia.sdk.predict import AsyncNucliaPredict, NucliaPredict


def test_predict(testing_config):
    np = NucliaPredict()
    embed = np.sentence(text="This is my text", model="multilingual-2024-05-06")
    assert embed.time > 0
    assert len(embed.data) == 1024


def test_predict_query(testing_config):
    np = NucliaPredict()
    query = np.query(
        text="Ramon, this is my text",
        semantic_model="multilingual-2024-05-06",
        token_model="multilingual",
        generative_model="chatgpt-azure-3",
    )
    assert query.language == "en"
    assert query.visual_llm is False
    assert query.max_context == 16385
    assert query.entities and query.entities.tokens[0].text == "Ramon"
    assert query.sentence and len(query.sentence.data) == 1024


def test_rag(testing_config):
    np = NucliaPredict()
    generated = np.rag(
        question="Which is the CEO of Nuclia?",
        context=[
            "Nuclia CTO is Ramon Navarro",
            "Eudald Camprubí is CEO at the same company as Ramon Navarro",
        ],
        model="chatgpt-azure-3",
    )
    assert "Eudald" in generated.answer


def test_generative(testing_config):
    np = NucliaPredict()
    generated = np.generate(text="How much is 2 + 2?", model="chatgpt-azure-3")
    assert "4" in generated.answer


async def test_async_generative(testing_config):
    np = AsyncNucliaPredict()
    generated = await np.generate(text="How much is 2 + 2?", model="chatgpt-azure-3")
    assert "4" in generated.answer


def test_stream_generative(testing_config):
    np = NucliaPredict()
    generated = np.generate_stream(text="How much is 2 + 2?", model="chatgpt-azure-3")
    assert "4" in generated.text


async def test_async_stream_generative(testing_config):
    np = AsyncNucliaPredict()
    generated = await np.generate_stream(
        text="How much is 2 + 2?", model="chatgpt-azure-3"
    )
    assert "4" in generated.text



from typing import List, Optional
from warnings import warn

from nucliadb_models.labels import KnowledgeBoxLabels, Label, LabelSet, LabelSetKind
from nucliadb_models.resource import Resource, ResourceList
from nucliadb_models.search import SummarizeRequest, SummaryKind

from nuclia.data import get_async_auth, get_auth
from nuclia.decorators import kb, pretty
from nuclia.lib.kb import AsyncNucliaDBClient, NucliaDBClient
from nuclia.lib.nua_responses import SummarizedModel
from nuclia.sdk.auth import AsyncNucliaAuth, NucliaAuth
from nuclia.sdk.export_import import (
    AsyncNucliaExports,
    AsyncNucliaImports,
    NucliaExports,
    NucliaImports,
)
from nuclia.sdk.logger import logger
from nuclia.sdk.logs import NucliaLogs
from nuclia.sdk.resource import AsyncNucliaResource, NucliaResource
from nuclia.sdk.search import AsyncNucliaSearch, NucliaSearch
from nuclia.sdk.upload import AsyncNucliaUpload, NucliaUpload


class NucliaKB:
    @property
    def _auth(self) -> NucliaAuth:
        auth = get_auth()
        return auth

    def __init__(self):
        self.upload = NucliaUpload()
        self.search = NucliaSearch()
        self.resource = NucliaResource()
        self.exports = NucliaExports()
        self.imports = NucliaImports()
        self.logs = NucliaLogs()

    @kb
    def list(self, *, interactive: bool = True, **kwargs) -> Optional[ResourceList]:
        ndb: NucliaDBClient = kwargs["ndb"]
        data: ResourceList = ndb.ndb.list_resources(kbid=ndb.kbid)
        if interactive:
            for resource in data.resources:
                status = (
                    resource.metadata.status if resource.metadata is not None else ""
                )
                print(f"{resource.id} {resource.icon:30} {resource.title} - {status}")
            return None
        else:
            return data

    @kb
    def get_labelset(
        self,
        *,
        labelset: str,
        **kwargs,
    ) -> LabelSet:
        ndb: NucliaDBClient = kwargs["ndb"]
        return ndb.ndb.get_labelset(kbid=ndb.kbid, labelset=labelset)

    @kb
    def add_labelset(
        self,
        *,
        labelset: str,
        kind: LabelSetKind = LabelSetKind.RESOURCES,
        multiple: bool = True,
        title: Optional[str] = None,
        color: Optional[str] = None,
        labels: Optional[List[str]] = None,
        **kwargs,
    ):
        ndb: NucliaDBClient = kwargs["ndb"]
        if labels is None:
            labels_list = []
        else:
            labels_list = [Label(title=label) for label in labels]

        labelset_obj = LabelSet(
            title=title if title is not None else labelset,
            color=color if color is not None else "blue",
            labels=labels_list,
            kind=[kind],
            multiple=multiple,
        )

        ndb.ndb.set_labelset(kbid=ndb.kbid, labelset=labelset, content=labelset_obj)

    @kb
    def list_labelsets(self, **kwargs) -> KnowledgeBoxLabels:
        ndb: NucliaDBClient = kwargs["ndb"]
        data: KnowledgeBoxLabels = ndb.ndb.get_labelsets(kbid=ndb.kbid)
        return data

    @kb
    def del_labelset(self, *, labelset: str, **kwargs):
        ndb: NucliaDBClient = kwargs["ndb"]
        ndb.ndb.delete_labelset(kbid=ndb.kbid, labelset=labelset)

    @kb
    def add_label(
        self,
        *,
        labelset: str,
        label: str,
        text: Optional[str] = None,
        uri: Optional[str] = None,
        **kwargs,
    ):
        ndb: NucliaDBClient = kwargs["ndb"]
        labelset_obj: LabelSet = ndb.ndb.get_labelset(kbid=ndb.kbid, labelset=labelset)
        label_obj = Label(title=label, text=text, uri=uri)
        labelset_obj.labels.append(label_obj)
        ndb.ndb.set_labelset(kbid=ndb.kbid, labelset=labelset, content=labelset_obj)

    @kb
    def del_label(self, *, labelset: str, label: str, **kwargs):
        ndb: NucliaDBClient = kwargs["ndb"]
        labelset_obj: LabelSet = ndb.ndb.get_labelset(kbid=ndb.kbid, labelset=labelset)
        label_to_delete = next(x for x in labelset_obj.labels if x.title == label)
        labelset_obj.labels.remove(label_to_delete)
        ndb.ndb.set_labelset(kbid=ndb.kbid, labelset=labelset, content=labelset_obj)

    @kb
    def set_configuration(
        self,
        *,
        semantic_model: Optional[str] = None,
        generative_model: Optional[str] = None,
        ner_model: Optional[str] = None,
        anonymization_model: Optional[str] = None,
        visual_labeling: Optional[str] = None,
        **kwargs,
    ):
        ndb: NucliaDBClient = kwargs["ndb"]
        ndb.ndb.set_configuration(
            kbid=ndb.kbid,
            semantic_model=semantic_model,
            generative_model=generative_model,
            ner_model=ner_model,
            anonymization_model=anonymization_model,
            visual_labeling=visual_labeling,
        )

    @kb
    @pretty
    def get_resource_by_id(self, *, rid: str, **kwargs) -> Resource:
        warn(
            "get_resource_by_id is deprecated, use resource.get instead",
            DeprecationWarning,
        )
        logger.warning("get_resource_by_slug is deprecated, use resource.get instead")
        ndb: NucliaDBClient = kwargs["ndb"]
        return ndb.ndb.get_resource_by_id(
            kbid=ndb.kbid, rid=rid, query_params={"show": "values"}
        )

    @kb
    @pretty
    def get_resource_by_slug(self, *, slug: str, **kwargs) -> Resource:
        warn(
            "get_resource_by_slug is deprecated, use resource.get instead",
            DeprecationWarning,
        )
        logger.warning("get_resource_by_slug is deprecated, use resource.get instead")
        ndb: NucliaDBClient = kwargs["ndb"]
        return ndb.ndb.get_resource_by_slug(
            kbid=ndb.kbid, slug=slug, query_params={"show": "values"}
        )

    @kb
    def delete(self, *, rid: str, **kwargs):
        warn("delete is deprecated, use resource.delete instead", DeprecationWarning)
        logger.warning("delete is deprecated, use resource.delete instead")
        ndb: NucliaDBClient = kwargs["ndb"]
        ndb.ndb.delete_resource(kbid=ndb.kbid, rid=rid)

    @kb
    def summarize(self, *, resources: List[str], **kwargs):
        ndb: NucliaDBClient = kwargs["ndb"]
        return ndb.ndb.summarize(kbid=ndb.kbid, resources=resources)


class AsyncNucliaKB:
    @property
    def _auth(self) -> AsyncNucliaAuth:
        auth = get_async_auth()
        return auth

    def __init__(self):
        self.upload = AsyncNucliaUpload()
        self.search = AsyncNucliaSearch()
        self.resource = AsyncNucliaResource()
        self.exports = AsyncNucliaExports()
        self.imports = AsyncNucliaImports()

    @kb
    async def list(
        self, *, interactive: bool = True, **kwargs
    ) -> Optional[ResourceList]:
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        del kwargs["ndb"]
        data: ResourceList = await ndb.ndb.list_resources(
            kbid=ndb.kbid, query_params=kwargs
        )
        if interactive:
            for resource in data.resources:
                status = (
                    resource.metadata.status if resource.metadata is not None else ""
                )
                print(f"{resource.id} {resource.icon:30} {resource.title} - {status}")
            return None
        else:
            return data

    @kb
    async def get_labelset(
        self,
        *,
        labelset: str,
        **kwargs,
    ) -> LabelSet:
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        return await ndb.ndb.get_labelset(kbid=ndb.kbid, labelset=labelset)

    @kb
    async def add_labelset(
        self,
        *,
        labelset: str,
        kind: LabelSetKind = LabelSetKind.RESOURCES,
        multiple: bool = True,
        title: Optional[str] = None,
        color: Optional[str] = None,
        labels: Optional[List[str]] = None,
        **kwargs,
    ):
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        if labels is None:
            labels_list = []
        else:
            labels_list = [Label(title=label) for label in labels]

        labelset_obj = LabelSet(
            title=title if title is not None else labelset,
            color=color if color is not None else "blue",
            labels=labels_list,
            kind=[kind],
            multiple=multiple,
        )

        await ndb.ndb.set_labelset(
            kbid=ndb.kbid, labelset=labelset, content=labelset_obj
        )

    @kb
    async def list_labelsets(self, **kwargs) -> KnowledgeBoxLabels:
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        data: KnowledgeBoxLabels = await ndb.ndb.get_labelsets(kbid=ndb.kbid)
        return data

    @kb
    async def del_labelset(self, *, labelset: str, **kwargs):
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        await ndb.ndb.delete_labelset(kbid=ndb.kbid, labelset=labelset)

    @kb
    async def add_label(
        self,
        *,
        labelset: str,
        label: str,
        text: Optional[str] = None,
        uri: Optional[str] = None,
        **kwargs,
    ):
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        labelset_obj: LabelSet = await ndb.ndb.get_labelset(
            kbid=ndb.kbid, labelset=labelset
        )
        label_obj = Label(title=label, text=text, uri=uri)
        labelset_obj.labels.append(label_obj)
        await ndb.ndb.set_labelset(
            kbid=ndb.kbid, labelset=labelset, content=labelset_obj
        )

    @kb
    async def del_label(self, *, labelset: str, label: str, **kwargs):
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        labelset_obj: LabelSet = await ndb.ndb.get_labelset(
            kbid=ndb.kbid, labelset=labelset
        )
        label_to_delete = next(x for x in labelset_obj.labels if x.title == label)
        labelset_obj.labels.remove(label_to_delete)
        await ndb.ndb.set_labelset(
            kbid=ndb.kbid, labelset=labelset, content=labelset_obj
        )

    @kb
    async def set_configuration(
        self,
        *,
        semantic_model: Optional[str] = None,
        generative_model: Optional[str] = None,
        ner_model: Optional[str] = None,
        anonymization_model: Optional[str] = None,
        visual_labeling: Optional[str] = None,
        **kwargs,
    ):
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        await ndb.ndb.set_configuration(
            kbid=ndb.kbid,
            semantic_model=semantic_model,
            generative_model=generative_model,
            ner_model=ner_model,
            anonymization_model=anonymization_model,
            visual_labeling=visual_labeling,
        )

    @kb
    async def summarize(
        self,
        *,
        resources: List[str],
        generative_model: Optional[str] = None,
        summary_kind: Optional[str] = None,
        timeout: int = 1000,
        **kwargs,
    ) -> SummarizedModel:
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        resp = await ndb.summarize(
            SummarizeRequest(
                resources=resources,
                generative_model=generative_model,
                summary_kind=SummaryKind(summary_kind),
            ),
            timeout=timeout,
        )
        return SummarizedModel.parse_obj(resp.json())



import base64
import json
import webbrowser
from typing import Any, Dict, List, Optional, Tuple

from httpx import AsyncClient, Client, ConnectError
from prompt_toolkit import prompt

from nuclia import get_global_url, get_regional_url
from nuclia.config import (
    Account,
    Config,
    KnowledgeBox,
    Zone,
    retrieve_account,
    retrieve_nua,
)
from nuclia.exceptions import NeedUserToken, UserTokenExpired

USER = "/api/v1/user/welcome"
MEMBER = "/api/v1/user"
ACCOUNTS = "/api/v1/accounts"
ZONES = "/api/v1/zones"
LIST_KBS = "/api/v1/account/{account}/kbs"
VERIFY_NUA = "/api/authorizer/info"


class BaseNucliaAuth:
    _inner_config: Optional[Config] = None

    @property
    def _config(self) -> Config:
        if self._inner_config is None:
            from nuclia.data import get_config

            self._inner_config = get_config()
        return self._inner_config

    def get_account_id(self, account_slug: str) -> str:
        account_obj = retrieve_account(self._config.accounts or [], account_slug)
        if not account_obj:
            raise ValueError(f"Account {account_slug} not found")
        return account_obj.id

    def list_nuas(self):
        result = []
        result.extend(
            self._config.nuas_token if self._config.nuas_token is not None else []
        )
        return result

    def default_nua(self, nua: str):
        nuas = self._config.nuas_token if self._config.nuas_token is not None else []
        nua_obj = retrieve_nua(nuas, nua)

        if nua_obj is None:
            raise KeyError("NUA KEY not found")
        self._config.set_default_nua(nua_obj.client_id)
        self._config.save()

    def default_kb(self, kbid: str):
        self._config.set_default_kb(kbid=kbid)

    def unset_kb(self, kbid: str):
        self._config.unset_default_kb(kbid=kbid)


class NucliaAuth(BaseNucliaAuth):
    client = Client()

    def show(self):
        if self._config.default:
            if self._config.default.account:
                print("Default Account")
                print("===============")
                print()
                print(self._config.default.account)
                print()
            if self._config.default.kbid:
                print("Default Knowledge Box")
                print("=====================")
                print()
                print(self._config.get_kb(self._config.default.kbid))
                print()
            if self._config.default.nua:
                print("Default NUA")
                print("===========")
                print()
                print(self._config.get_nua(self._config.default.nua))
                print()
            if self._config.default.nucliadb:
                print("Default NUCLIADB")
                print("================")
                print()
                print(self._config.default.nucliadb)
                print()

        if self._config.token:
            print("User Auth")
            print("=========")
            print()
            try:
                self._show_user()
            except Exception:
                print("Not Authenticated")
            print()

        if len(self._config.kbs_token):
            print("Knowledgebox Auth")
            print("=================")
            print()
            for kb in self._config.kbs_token:
                print(kb)
            print()

        if len(self._config.nuas_token):
            print("NUA Key")
            print("=======")
            print()
            for nua in self._config.nuas_token:
                print(nua)

    def nucliadb(self, url: str = "http://localhost:8080"):
        """
        Setup a local NucliaDB. Needs to be the base url of the NucliaDB server
        """
        resp = self.client.get(url)
        if resp.status_code != 200 or b"Nuclia" not in resp.content:
            raise Exception("Not a valid URL")
        self._config.set_default_nucliadb(nucliadb=url)

    def kb(self, url: str, token: Optional[str] = None) -> Optional[str]:
        url = url.strip("/")
        kbid, title = self.validate_kb(url, token)
        if kbid:
            print("Validated")
            self._config.set_kb_token(url=url, token=token, title=title, kbid=kbid)
            self._config.set_default_kb(kbid=kbid)
        else:
            print("Invalid service token")
        return kbid

    def nua(self, token: str) -> Optional[str]:
        client_id, account_type, account, base_region = self.validate_nua(token)
        if account is not None and client_id is not None and base_region is not None:
            print("Validated")
            self._config.set_nua_token(
                client_id=client_id,
                account_type=account_type,
                account=account,
                base_region=base_region,
                token=token,
            )
            return client_id
        else:
            print("Invalid NUA token")
            return None

    def validate_nua(
        self, token: str
    ) -> Tuple[Optional[str], Optional[str], Optional[str], Optional[str]]:
        # Validate the code is ok
        token_payload = token.split(".")[1]
        token_payload_decoded = str(base64.b64decode(token_payload + "=="), "utf-8")
        payload = json.loads(token_payload_decoded)
        base_path = payload["iss"]
        url = base_path.strip("/") + VERIFY_NUA
        resp = self.client.get(
            url,
            headers={"x-nuclia-nuakey": f"Bearer {token}"},
        )
        if resp.status_code == 200:
            data = resp.json().get("user")
            return (
                data.get("user_id"),
                data.get("account_type"),
                data.get("account_id"),
                payload.get("iss"),
            )
        else:
            return None, None, None, None

    def validate_kb(
        self, url: str, token: Optional[str] = None
    ) -> Tuple[Optional[str], Optional[str]]:
        # Validate the code is ok
        if token is None:
            # Validate OSS version
            resp = self.client.get(
                url,
                headers={"X-NucliaDB-ROLES": f"READER"},
            )
        else:
            # Validate Cloud version
            resp = self.client.get(
                url,
                headers={"X-Nuclia-Serviceaccount": f"Bearer {token}"},
            )
        if resp.status_code == 200:
            data = resp.json()
            return data.get("uuid"), data.get("config", {}).get("title")
        else:
            return None, None

    def _show_user(self):
        resp = self._request("GET", get_global_url(MEMBER))
        assert resp
        print(f"User: {resp.get('name')} <{resp.get('email')}>")
        print(f"Type: {resp.get('type')}")

    def login(self):
        """
        Lets redirect the user to the UI to capture a token
        """
        if self._validate_user_token():
            print("Logged in!")
            self._show_user()
            return

        webbrowser.open(get_global_url("/redirect?display=token"))
        # we cannot use Python's `input` here because the copy/pasted token is too long
        code = prompt("Follow the browser flow and copy the token and paste it here:")
        print("Checking...")
        self.set_user_token(code)

    def logout(self):
        """
        Remove the current user token.
        """
        self._config.remove_user_token()

    def set_user_token(self, code: str):
        if self._validate_user_token(code):
            self._config.set_user_token(code)
            print("Auth completed!")
            self.post_login()
        else:
            print("Invalid token auth not completed")

    def _extract_account(self, token: str) -> str:
        base64url = token.split(".")[1]
        data = json.loads(
            base64.urlsafe_b64decode(base64url + "=" * (4 - len(base64url) % 4))
        )
        return data.get("jti")

    def _validate_user_token(self, code: Optional[str] = None) -> bool:
        # Validate the code is ok
        if code is None:
            code = self._config.token
        resp = self.client.get(
            get_global_url(USER),
            headers={"Authorization": f"Bearer {code}"},
        )
        if resp.status_code == 200:
            return True
        else:
            return False

    def post_login(self):
        self.accounts()
        self.zones()

    def _request(
        self, method: str, path: str, data: Optional[Any] = None, remove_null=True
    ):
        if not self._config.token:
            raise NeedUserToken()
        kwargs: Dict[str, Any] = {
            "headers": {"Authorization": f"Bearer {self._config.token}"}
        }
        if data is not None:
            if remove_null:
                data = {k: v for k, v in data.items() if v is not None}
            kwargs["data"] = json.dumps(data)
        resp = self.client.request(
            method,
            path,
            **kwargs,
        )
        if resp.status_code == 204:
            return None
        elif resp.status_code >= 200 and resp.status_code < 300:
            return resp.json()
        elif resp.status_code >= 300 and resp.status_code < 400:
            return None
        elif resp.status_code == 403:
            raise UserTokenExpired()
        else:
            raise Exception({"status": resp.status_code, "message": resp.text})

    def accounts(self) -> List[Account]:
        accounts = self._request("GET", get_global_url(ACCOUNTS))
        result: List[Account] = []
        self._config.accounts = []
        if accounts is None:
            return result
        for account in accounts:
            account_obj = Account.parse_obj(account)
            result.append(account_obj)
            self._config.accounts.append(account_obj)
        self._config.save()
        return result

    def zones(self) -> List[Zone]:
        zones = self._request("GET", get_global_url(ZONES))
        if self._config.accounts is None:
            self._config.accounts = []
        self._config.zones = []
        result: List[Zone] = []
        if zones is None:
            return result
        for zone in zones:
            zone_obj = Zone.parse_obj(zone)
            result.append(zone_obj)
            self._config.zones.append(zone_obj)
        self._config.save()
        return result

    def kbs(self, account: str) -> List[KnowledgeBox]:
        result: List[KnowledgeBox] = []
        zones = self.zones()
        for zoneObj in zones:
            zoneSlug = zoneObj.slug
            if not zoneSlug:
                continue
            path = get_regional_url(zoneSlug, LIST_KBS.format(account=account))
            try:
                kbs = self._request("GET", path)
            except UserTokenExpired:
                return []
            except ConnectError:
                print(
                    f"Connection error to {get_regional_url(zoneSlug, '')}, skipping zone"
                )
                continue
            if kbs is not None:
                for kb in kbs:
                    url = get_regional_url(zoneSlug, f"/api/v1/kb/{kb['id']}")
                    kb_obj = KnowledgeBox(
                        url=url,
                        id=kb["id"],
                        slug=kb["slug"],
                        title=kb["title"],
                        account=account,
                        region=zoneSlug,
                    )
                    result.append(kb_obj)
        return result


class AsyncNucliaAuth(BaseNucliaAuth):
    client = AsyncClient()

    async def show(self):
        if self._config.default:
            if self._config.default.account:
                print("Default Account")
                print("===============")
                print()
                print(self._config.default.account)
                print()
            if self._config.default.kbid:
                print("Default Knowledge Box")
                print("=====================")
                print()
                print(self._config.get_kb(self._config.default.kbid))
                print()

        if self._config.token:
            print("User Auth")
            print("=========")
            print()
            try:
                await self._show_user()
            except Exception:
                print("Not Authenticated")
            print()

        if len(self._config.kbs_token):
            print("Knowledgebox Auth")
            print("=================")
            print()
            for kb in self._config.kbs_token:
                print(kb)
            print()

        if len(self._config.nuas_token):
            print("NUA Key")
            print("=======")
            print()
            for nua in self._config.nuas_token:
                print(nua)

    async def nucliadb(self, url: str = "http://localhost:8080"):
        """
        Setup a local NucliaDB. Needs to be the base url of the NucliaDB server
        """
        resp = await self.client.get(url)
        if resp.status_code != 200 or b"Nuclia" not in resp.content:
            raise Exception("Not a valid URL")
        self._config.set_default_nucliadb(nucliadb=url)

    async def kb(self, url: str, token: Optional[str] = None) -> bool:
        url = url.strip("/")
        kbid, title = await self.validate_kb(url, token)
        if kbid:
            print("Validated")
            self._config.set_kb_token(url=url, token=token, title=title, kbid=kbid)
            self._config.set_default_kb(kbid=kbid)
            return True
        else:
            print("Invalid service token")
            return False

    async def nua(self, token: str) -> Optional[str]:
        client_id, account_type, account, base_region = await self.validate_nua(token)
        if account is not None and client_id is not None and base_region is not None:
            print("Validated")
            self._config.set_nua_token(
                client_id=client_id,
                account_type=account_type,
                account=account,
                base_region=base_region,
                token=token,
            )
            return client_id
        else:
            print("Invalid NUA token")
            return None

    async def validate_nua(
        self, token: str
    ) -> Tuple[Optional[str], Optional[str], Optional[str], Optional[str]]:
        # Validate the code is ok
        token_payload = token.split(".")[1]
        token_payload_decoded = str(base64.b64decode(token_payload + "=="), "utf-8")
        payload = json.loads(token_payload_decoded)
        base_path = payload["iss"]
        url = base_path.strip("/") + VERIFY_NUA
        resp = await self.client.get(
            url,
            headers={"x-nuclia-nuakey": f"Bearer {token}"},
        )
        if resp.status_code == 200:
            data = resp.json().get("user")
            return (
                data.get("user_id"),
                data.get("account_type"),
                data.get("account_id"),
                payload.get("iss"),
            )
        else:
            return None, None, None, None

    async def validate_kb(
        self, url: str, token: Optional[str] = None
    ) -> Tuple[Optional[str], Optional[str]]:
        # Validate the code is ok
        if token is None:
            # Validate OSS version
            resp = await self.client.get(
                url,
                headers={"X-NucliaDB-ROLES": f"READER"},
            )
        else:
            # Validate Cloud version
            resp = await self.client.get(
                url,
                headers={"X-Nuclia-Serviceaccount": f"Bearer {token}"},
            )
        if resp.status_code == 200:
            data = resp.json()
            return data.get("uuid"), data.get("config", {}).get("title")
        else:
            return None, None

    async def _show_user(self):
        resp = await self._request("GET", get_global_url(MEMBER))
        assert resp
        print(f"User: {resp.get('name')} <{resp.get('email')}>")
        print(f"Type: {resp.get('type')}")

    async def login(self):
        """
        Lets redirect the user to the UI to capture a token
        """
        if await self._validate_user_token():
            print("Logged in!")
            await self._show_user()
            return

        webbrowser.open(get_global_url("/redirect?display=token"))
        # we cannot use Python's `input` here because the copy/pasted token is too long
        code = prompt("Follow the browser flow and copy the token and paste it here:")
        print("Checking...")
        await self.set_user_token(code)

    def logout(self):
        """
        Remove the current user token.
        """
        self._config.remove_user_token()

    async def set_user_token(self, code: str):
        if await self._validate_user_token(code):
            self._config.set_user_token(code)
            print("Auth completed!")
            await self.post_login()
        else:
            print("Invalid token auth not completed")

    def _extract_account(self, token: str) -> str:
        base64url = token.split(".")[1]
        data = json.loads(
            base64.urlsafe_b64decode(base64url + "=" * (4 - len(base64url) % 4))
        )
        return data.get("jti")

    async def _validate_user_token(self, code: Optional[str] = None) -> bool:
        # Validate the code is ok
        if code is None:
            code = self._config.token
        resp = await self.client.get(
            get_global_url(USER),
            headers={"Authorization": f"Bearer {code}"},
        )
        if resp.status_code == 200:
            return True
        else:
            return False

    async def post_login(self):
        await self.accounts()
        await self.zones()

    async def _request(
        self, method: str, path: str, data: Optional[Any] = None, remove_null=True
    ):
        if not self._config.token:
            raise NeedUserToken()
        kwargs: Dict[str, Any] = {
            "headers": {"Authorization": f"Bearer {self._config.token}"}
        }
        if data is not None:
            if remove_null:
                data = {k: v for k, v in data.items() if v is not None}
            kwargs["data"] = json.dumps(data)
        resp = await self.client.request(
            method,
            path,
            **kwargs,
        )
        if resp.status_code == 204:
            return None
        elif resp.status_code >= 200 and resp.status_code < 300:
            return resp.json()
        elif resp.status_code >= 300 and resp.status_code < 400:
            return None
        elif resp.status_code == 403:
            raise UserTokenExpired()
        else:
            raise Exception({"status": resp.status_code, "message": resp.text})

    async def accounts(self) -> List[Account]:
        accounts = await self._request("GET", get_global_url(ACCOUNTS))
        result: List[Account] = []
        self._config.accounts = []
        if accounts is None:
            return result
        for account in accounts:
            account_obj = Account.parse_obj(account)
            result.append(account_obj)
            self._config.accounts.append(account_obj)
        self._config.save()
        return result

    async def zones(self) -> List[Zone]:
        zones = await self._request("GET", get_global_url(ZONES))
        if self._config.accounts is None:
            self._config.accounts = []
        self._config.zones = []
        result: List[Zone] = []
        if zones is None:
            return result
        for zone in zones:
            zone_obj = Zone.parse_obj(zone)
            result.append(zone_obj)
            self._config.zones.append(zone_obj)
        self._config.save()
        return result

    async def kbs(self, account: str) -> List[KnowledgeBox]:
        result: List[KnowledgeBox] = []
        zones = await self.zones()
        for zoneObj in zones:
            zoneSlug = zoneObj.slug
            if not zoneSlug:
                continue
            path = get_regional_url(zoneSlug, LIST_KBS.format(account=account))
            try:
                kbs = await self._request("GET", path)
            except UserTokenExpired:
                return result
            except ConnectionError:
                print(
                    f"Connection error to {get_regional_url(zoneSlug, '')}, skipping zone"
                )
                continue
            if kbs is not None:
                for kb in kbs:
                    url = get_regional_url(zoneSlug, f"/api/v1/kb/{kb['id']}")
                    kb_obj = KnowledgeBox(
                        url=url,
                        id=kb["id"],
                        slug=kb["slug"],
                        title=kb["title"],
                        account=account,
                        region=zoneSlug,
                    )
                    result.append(kb_obj)
        return result



from nuclia.data import get_auth
from nuclia.sdk.auth import NucliaAuth


class NucliaAccounts:
    @property
    def _auth(self) -> NucliaAuth:
        return get_auth()

    def list(self):
        return self._auth.accounts()

    def default(self, account: str):
        accounts = (
            self._auth._config.accounts
            if self._auth._config.accounts is not None
            else []
        )
        slugs = [account_obj.slug for account_obj in accounts]
        if account not in slugs:
            raise KeyError("Account not found")
        self._auth._config.set_default_account(account)
        self._auth._config.save()



from typing import Union

import httpx
import requests

from nuclia.exceptions import RateLimitError, UserTokenExpired


def handle_http_errors(response: Union[httpx.Response, requests.models.Response]):
    if (
        response.status_code == 403
        and "Hydra token is either unexistent or revoked" in response.text
    ):
        raise UserTokenExpired()
    elif response.status_code == 429:
        raise RateLimitError(f"Rate limited: {response.text}")
    elif response.status_code >= 400:
        raise httpx.HTTPError(f"Status code {response.status_code}: {response.text}")



import sys
import warnings
from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union

from nucliadb_models.search import (
    AskRequest,
    AskResponseItem,
    Filter,
    FindRequest,
    KnowledgeboxFindResults,
    KnowledgeboxSearchResults,
    Relations,
    SearchOptions,
    SearchRequest,
    SyncAskResponse,
)
from pydantic import ValidationError

from nuclia.data import get_async_auth, get_auth
from nuclia.decorators import kb, pretty
from nuclia.lib.kb import AsyncNucliaDBClient, NucliaDBClient
from nuclia.sdk.auth import AsyncNucliaAuth, NucliaAuth


@dataclass
class AskAnswer:
    answer: bytes
    learning_id: str
    relations_result: Optional[Relations]
    find_result: Optional[KnowledgeboxFindResults]
    citations: Optional[Dict[str, Any]]
    timings: Optional[Dict[str, float]]
    tokens: Optional[Dict[str, int]]

    def __str__(self):
        return self.answer.decode()


class NucliaSearch:
    """
    Perform search on a Knowledge Box.

    `find` and `search` accept the following parameters:
    - `json`: return results in JSON format
    - `yaml`: return results in YAML format
    """

    @property
    def _auth(self) -> NucliaAuth:
        auth = get_auth()
        return auth

    @kb
    @pretty
    def search(
        self,
        *,
        query: Union[str, SearchRequest] = "",
        filters: Optional[Union[List[str], List[Filter]]] = None,
        **kwargs,
    ) -> KnowledgeboxSearchResults:
        """
        Perform a search query.

        See https://docs.nuclia.dev/docs/api#tag/Search/operation/Search_Knowledge_Box_kb__kbid__search_post
        """
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        if isinstance(query, str):
            req = SearchRequest(query=query, filters=(filters or []))  # type: ignore
        elif isinstance(query, SearchRequest):
            req = query
        elif isinstance(query, dict):
            try:
                req = SearchRequest.parse_obj(query)
            except ValidationError as exc:
                print(exc)
                sys.exit(1)
        else:
            raise Exception("Invalid Query either str or SearchRequest")

        return ndb.ndb.search(req, kbid=ndb.kbid)

    @kb
    @pretty
    def find(
        self,
        *,
        query: Union[str, FindRequest] = "",
        highlight: Optional[bool] = False,
        relations: Optional[bool] = False,
        filters: Optional[Union[List[str], List[Filter]]] = None,
        **kwargs,
    ):
        """
        Perform a find query.

        See https://docs.nuclia.dev/docs/api#tag/Search/operation/Find_Knowledge_Box_kb__kbid__find_post
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        if isinstance(query, str) and highlight is not None:
            req = FindRequest(
                query=query,
                highlight=highlight,
                filters=filters or [],  # type: ignore
            )
        elif isinstance(query, FindRequest):
            req = query
        elif isinstance(query, dict):
            try:
                req = FindRequest.parse_obj(query)
            except ValidationError as exc:
                print(exc)
                sys.exit(1)
        else:
            raise Exception("Invalid Query either str or FindRequest")

        if relations:
            req.features.append(SearchOptions.RELATIONS)

        return ndb.ndb.find(req, kbid=ndb.kbid)

    @kb
    def ask(
        self,
        *,
        query: Union[str, dict, AskRequest],
        filters: Optional[Union[List[str], List[Filter]]] = None,
        **kwargs,
    ):
        """
        Answer a question.

        See https://docs.nuclia.dev/docs/api#tag/Search/operation/Ask_Knowledge_Box_kb__kbid__ask_post
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        if isinstance(query, str):
            req = AskRequest(
                query=query,
                filters=filters or [],  # type: ignore
            )
        elif isinstance(query, dict):
            try:
                req = AskRequest.parse_obj(query)
            except ValidationError as exc:
                print(exc)
                sys.exit(1)
        elif isinstance(query, AskRequest):
            req = query
        else:
            raise ValueError("Invalid query type. Must be str, dict or AskRequest.")

        ask_response: SyncAskResponse = ndb.ndb.ask(kbid=ndb.kbid, content=req)

        result = AskAnswer(
            answer=ask_response.answer.encode(),
            learning_id=ask_response.learning_id,
            relations_result=ask_response.relations,
            find_result=ask_response.retrieval_results,
            citations=ask_response.citations,
            timings=None,
            tokens=None,
        )
        if ask_response.metadata is not None:
            if ask_response.metadata.timings is not None:
                result.timings = ask_response.metadata.timings.dict()
            if ask_response.metadata.tokens is not None:
                result.tokens = ask_response.metadata.tokens.dict()
        return result


class AsyncNucliaSearch:
    """
    Perform search on a Knowledge Box.

    `find` and `search` accept the following parameters:
    - `json`: return results in JSON format
    - `yaml`: return results in YAML format
    """

    @property
    def _auth(self) -> AsyncNucliaAuth:
        auth = get_async_auth()
        return auth

    @kb
    @pretty
    async def search(
        self,
        *,
        query: Union[str, SearchRequest] = "",
        filters: Optional[List[str]] = None,
        **kwargs,
    ):
        """
        Perform a search query.

        See https://docs.nuclia.dev/docs/api#tag/Search/operation/Search_Knowledge_Box_kb__kbid__search_post
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        if isinstance(query, str):
            req = SearchRequest(query=query, filters=(filters or []))
        elif isinstance(query, SearchRequest):
            req = query
        elif isinstance(query, dict):
            try:
                req = SearchRequest.parse_obj(query)
            except ValidationError as exc:
                print(exc)
                sys.exit(1)
        else:
            raise Exception("Invalid Query either str or SearchRequest")

        return await ndb.ndb.search(req, kbid=ndb.kbid)

    @kb
    @pretty
    async def find(
        self,
        *,
        query: Union[str, FindRequest] = "",
        highlight: Optional[bool] = False,
        relations: Optional[bool] = False,
        filters: Optional[List[str]] = None,
        **kwargs,
    ) -> KnowledgeboxFindResults:
        """
        Perform a find query.

        See https://docs.nuclia.dev/docs/api#tag/Search/operation/Find_Knowledge_Box_kb__kbid__find_post
        """

        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        if isinstance(query, str) and highlight is not None:
            req = FindRequest(query=query, highlight=highlight, filters=(filters or []))
        elif isinstance(query, FindRequest):
            req = query
        elif isinstance(query, dict):
            try:
                req = FindRequest.parse_obj(query)
            except ValidationError as exc:
                print(exc)
                sys.exit(1)
        else:
            raise Exception("Invalid Query either str or FindRequest")

        if relations:
            req.features.append(SearchOptions.RELATIONS)

        return await ndb.ndb.find(req, kbid=ndb.kbid)

    @kb
    async def ask(
        self,
        *,
        query: Union[str, dict, AskRequest],
        filters: Optional[List[str]] = None,
        timeout: int = 100,
        **kwargs,
    ):
        """
        Answer a question.

        See https://docs.nuclia.dev/docs/api#tag/Search/operation/Ask_Knowledge_Box_kb__kbid__ask_post
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        if isinstance(query, str):
            req = AskRequest(
                query=query,
                filters=filters or [],  # type: ignore
            )
        elif isinstance(query, dict):
            try:
                req = AskRequest.parse_obj(query)
            except ValidationError as exc:
                print(exc)
                sys.exit(1)
        elif isinstance(query, AskRequest):
            req = query
        else:
            raise ValueError("Invalid query type. Must be str, dict or AskRequest.")
        ask_stream_response = await ndb.ask(req, timeout=timeout)
        result = AskAnswer(
            answer=b"",
            learning_id=ask_stream_response.headers.get("NUCLIA-LEARNING-ID", ""),
            relations_result=None,
            find_result=None,
            citations=None,
            timings=None,
            tokens=None,
        )
        async for line in ask_stream_response.aiter_lines():
            try:
                ask_response_item = AskResponseItem.parse_raw(line).item
            except Exception as e:
                warnings.warn(f"Failed to parse AskResponseItem: {e}. item: {line}")
                continue
            if ask_response_item.type == "answer":
                result.answer += ask_response_item.text.encode()
            elif ask_response_item.type == "retrieval":
                result.find_result = ask_response_item.results
            elif ask_response_item.type == "relations":
                result.relations_result = ask_response_item.relations
            elif ask_response_item.type == "citations":
                result.citations = ask_response_item.citations
            elif ask_response_item.type == "metadata":
                if ask_response_item.timings:
                    result.timings = ask_response_item.timings.dict()
                if ask_response_item.tokens:
                    result.tokens = ask_response_item.tokens.dict()
            elif ask_response_item.type == "status":
                # Status is ignored
                pass
            else:  # pragma: no cover
                warnings.warn(f"Unknown ask stream item type: {ask_response_item.type}")
        return result



from .accounts import NucliaAccounts  # noqa
from .agent import NucliaAgent  # noqa
from .auth import AsyncNucliaAuth  # noqa
from .auth import NucliaAuth  # noqa
from .export_import import AsyncNucliaExports  # noqa
from .export_import import NucliaExports, NucliaImports  # noqa
from .kb import NucliaKB  # noqa
from .kbs import NucliaKBS  # noqa
from .nua import NucliaNUA  # noqa
from .nucliadb import NucliaDB  # noqa
from .predict import AsyncNucliaPredict  # noqa
from .predict import NucliaPredict  # noqa
from .resource import AsyncNucliaResource  # noqa
from .resource import NucliaResource  # noqa
from .search import AsyncNucliaSearch  # noqa
from .search import NucliaSearch  # noqa
from .upload import AsyncNucliaUpload  # noqa
from .upload import NucliaUpload  # noqa
from .zones import NucliaZones  # noqa



from nuclia.sdk.kb import NucliaKB
from nuclia.tests.fixtures import IS_PROD


def test_logs(testing_config):
    if not IS_PROD:
        assert True
        return
    nkb = NucliaKB()
    logs = nkb.logs.get(type="SEARCH", month="2023-06")
    assert len(logs) == 16



from nuclia.sdk.agent import NucliaAgent


def test_agent(testing_config):
    np = NucliaAgent()
    agent_prompt = np.generate_prompt(
        text="You will answer questions about this topic in the way of a Spanish historian",
        agent_definition="Toledo",
        model="anthropic",
    )
    assert "Toledo" in agent_prompt



import asyncio
import os
import time
from functools import partial
from typing import Optional

import aiofiles
from nucliadb_models.export_import import (
    CreateExportResponse,
    CreateImportResponse,
    Status,
    StatusResponse,
)
from tqdm import tqdm

from nuclia.decorators import kb
from nuclia.lib.kb import AsyncNucliaDBClient, NucliaDBClient

MB = 1024 * 1024
CHUNK_SIZE = 10 * MB
STATUS_CHECK_INTERVAL_S = 3


class NucliaExports:
    """
    Manage Knowledge Box exports.

    To start an export and save it locally, run:

    nuclia kb exports start --path=$HOME/path/to/export
    """

    @kb
    def start(
        self, *args, path: Optional[str] = None, **kwargs
    ) -> Optional[CreateExportResponse]:
        """
        Start an export.

        :param path: if specified, the export contents will be downloaded there.
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        resp = ndb.ndb.start_export(kbid=ndb.kbid)
        if path is None:
            return resp
        print(f"Export for KB started. export_id={resp.export_id}")
        self.download(export_id=resp.export_id, path=path, **kwargs)
        return None

    @kb
    def download(self, *, export_id: str, path: str, **kwargs) -> None:
        """
        Download an already generated export.

        :param export_id: id of the export to download
        :param path: file where the export data will be saved
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        wait_for_task_to_finish(ndb, "export", export_id)
        print(f"Export is ready. Will be downloaded to {path}.")
        export_size = ndb.ndb.export_status(kbid=ndb.kbid, export_id=export_id).total
        iterator = ndb.ndb.download_export(kbid=ndb.kbid, export_id=export_id)
        with open(path, "wb") as f:
            with tqdm(
                desc=f"Downloading data",
                total=export_size,
                unit="iB",
                unit_scale=True,
            ) as pbar:
                for chunk in iterator(chunk_size=CHUNK_SIZE):
                    pbar.update(len(chunk))
                    f.write(chunk)


class AsyncNucliaExports:
    """
    Manage Knowledge Box exports.

    To start an export and save it locally, run:

    nuclia kb exports start --path=$HOME/path/to/export
    """

    @kb
    async def start(
        self, *args, path: Optional[str] = None, **kwargs
    ) -> Optional[CreateExportResponse]:
        """
        Start an export.

        :param path: if specified, the export contents will be downloaded there.
        """
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        resp = await ndb.ndb.start_export(kbid=ndb.kbid)
        if path is None:
            return resp
        print(f"Export for KB started. export_id={resp.export_id}")
        await self.download(export_id=resp.export_id, path=path, **kwargs)
        return None

    @kb
    async def download(self, *, export_id: str, path: str, **kwargs) -> None:
        """
        Download an already generated export.

        :param export_id: id of the export to download
        :param path: file where the export data will be saved
        """
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        await async_wait_for_task_to_finish(ndb, "export", export_id)
        print(f"Export is ready. Will be downloaded to {path}.")
        await ndb.download_export(path=path, export_id=export_id, chunk_size=CHUNK_SIZE)


class NucliaImports:
    """
    Manage Knowledge Box imports.

    """

    @kb
    def start(
        self, *, path: str, sync: bool = False, **kwargs
    ) -> Optional[CreateImportResponse]:
        """
        Start an import.

        :param path: path to the file with the export data.
        :param sync: waits for the server import task to finish.
        """
        ndb: NucliaDBClient = kwargs["ndb"]

        def iterator(path: str):
            total_size = os.path.getsize(path)
            with tqdm(
                desc=f"Uploading data",
                total=total_size,
                unit="iB",
                unit_scale=True,
            ) as pbar:
                with open(path, "rb") as f:
                    while True:
                        chunk = f.read(CHUNK_SIZE)
                        if not chunk:
                            break
                        pbar.update(len(chunk))
                        yield chunk

        print(f"Importing from {path}")
        response = ndb.ndb.start_import(kbid=ndb.kbid, content=iterator(path))

        if not sync:
            print(f"Import task started.")
            return response
        else:
            import_id = response.import_id
            print(f"Import task started. import_id={import_id}")
            wait_for_task_to_finish(ndb, "import", import_id)
            print(f"Import finished!")
            return None

    @kb
    def status(self, *, import_id: str, **kwargs) -> StatusResponse:
        """
        Check the status of an import.

        :param import_id: id of the import task.
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        return ndb.ndb.import_status(kbid=ndb.kbid, import_id=import_id)


class AsyncNucliaImports:
    """
    Manage Knowledge Box imports.

    """

    @kb
    async def start(
        self, *, path: str, sync: bool = False, **kwargs
    ) -> Optional[CreateImportResponse]:
        """
        Start an import.

        :param path: path to the file with the export data.
        :param sync: waits for the server import task to finish.
        """
        ndb: AsyncNucliaDBClient = kwargs["ndb"]

        async def iterator(path: str):
            total_size = os.path.getsize(path)
            with tqdm(
                desc=f"Uploading data",
                total=total_size,
                unit="iB",
                unit_scale=True,
            ) as pbar:
                async with aiofiles.open(path, "rb") as f:
                    while True:
                        chunk = await f.read(CHUNK_SIZE)
                        if not chunk:
                            break
                        pbar.update(len(chunk))
                        yield chunk

        print(f"Importing from {path}")
        response = await ndb.ndb.start_import(kbid=ndb.kbid, content=iterator(path))

        if not sync:
            print(f"Import task started.")
            return response
        else:
            import_id = response.import_id
            print(f"Import task started. import_id={import_id}")
            await async_wait_for_task_to_finish(ndb, "import", import_id)
            print(f"Import finished!")
            return None

    @kb
    def status(self, *, import_id: str, **kwargs) -> StatusResponse:
        """
        Check the status of an import.

        :param import_id: id of the import task.
        """
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        return ndb.ndb.import_status(kbid=ndb.kbid, import_id=import_id)


def wait_for_task_to_finish(ndb: NucliaDBClient, task_type: str, id: str):
    if task_type not in ("export", "import"):
        raise ValueError(f"Unknown task_type {task_type}")

    if task_type == "export":
        desc = "Generating export. Status: {status}"
        get_status = partial(ndb.ndb.export_status, kbid=ndb.kbid, export_id=id)
        unit = "resources"
        unit_scale = False

    else:
        desc = "Importing data. Status: {status}"
        get_status = partial(ndb.ndb.import_status, kbid=ndb.kbid, import_id=id)
        unit = "iB"
        unit_scale = True

    resp: StatusResponse = get_status()
    with tqdm(
        desc=desc.format(status=resp.status),
        unit=unit,
        total=resp.total,
        unit_scale=unit_scale,
    ) as pbar:
        status = resp.status
        processed = 0
        while status != Status.FINISHED:
            assert status != Status.ERRORED, f"{task_type} failed"

            # Update progress bar
            pbar.total = resp.total
            pbar.desc = desc.format(status=resp.status)
            delta_processed = max(0, resp.processed - processed)
            pbar.update(delta_processed)
            processed = resp.processed

            time.sleep(STATUS_CHECK_INTERVAL_S)

            resp = get_status()
            status = resp.status


async def async_wait_for_task_to_finish(
    ndb: AsyncNucliaDBClient, task_type: str, id: str
):
    if task_type not in ("export", "import"):
        raise ValueError(f"Unknown task_type {task_type}")

    if task_type == "export":
        desc = "Generating export. Status: {status}"
        get_status = partial(ndb.ndb.export_status, kbid=ndb.kbid, export_id=id)
        unit = "resources"
        unit_scale = False

    else:
        desc = "Importing data. Status: {status}"
        get_status = partial(ndb.ndb.import_status, kbid=ndb.kbid, import_id=id)
        unit = "iB"
        unit_scale = True

    resp: StatusResponse = await get_status()
    with tqdm(
        desc=desc.format(status=resp.status),
        unit=unit,
        total=resp.total,
        unit_scale=unit_scale,
    ) as pbar:
        status = resp.status
        processed = 0
        while status != Status.FINISHED:
            assert status != Status.ERRORED, f"{task_type} failed"

            # Update progress bar
            pbar.total = resp.total
            pbar.desc = desc.format(status=resp.status)
            delta_processed = max(0, resp.processed - processed)
            pbar.update(delta_processed)
            processed = resp.processed

            await asyncio.sleep(STATUS_CHECK_INTERVAL_S)

            resp = await get_status()
            status = resp.status



import warnings
from nucliadb_sdk.v2.exceptions import NotFoundError

from nuclia.sdk.resource import NucliaResource


def test_resource(testing_config):
    nresource = NucliaResource()
    try:
        res = nresource.get(slug="res1")
        nresource.delete(rid=res.id)
    except NotFoundError:
        pass

    res_id = nresource.create(slug="res1")

    res = nresource.get(rid=res_id)
    assert res

    res = nresource.get(slug="res1")
    assert res
    assert res.id == res_id

    assert res.title == "res1"

    nresource.update(
        rid=res_id,
        title="My great resource",
        texts={"text1": {"body": "Hello here"}},
    )
    res = nresource.get(slug="res1", show=["basic", "values"])

    assert res
    assert res.title in ["My great resource", "res1"]
    if res.data.texts:
        assert res.data.texts["text1"]
    else:
        warnings.warn("No texts found in resource")

    nresource.delete(rid=res_id)

    try:
        nresource.get(slug="res1")
        assert False
    except NotFoundError:
        assert True
